
function [dist, label_indeces] = chamfer_matching(template, distmap, label, dirmap, sc_map, max_dist)

	if nargin ~= 6 %#ok<ALIGN>
		max_dist = 10;
    end

    max_scale = 5;
   
    [rows, cols] = size(distmap); %#ok<NASGU>
    indeces = (template(:, 2) - 1) .* rows + template(:, 1);
    label_indeces = label(indeces(:));
	
    % chamfer macthing distance
	distance = distmap(indeces);
    help_id = distance > max_dist;
    distance(help_id) = max_dist;
    distance = sum(distance);
	distance = distance / (length(template) * max_dist);
    	
    % angle distance
	if ~isempty(dirmap)
		angle_diff = abs( template(:, 3) - dirmap( label_indeces ) );
		angle_diff = sum(angle_diff) / (length(template) * pi);	
	else
		angle_diff = 0.2;
	end
	
	% shape context dist
	if ~isempty(sc_map)
		sc_v1 = cell2mat( sc_map(label_indeces) );
		sc_v2 = template(:, 4:end);
		help_sc_dist = (sc_v1-sc_v2).^2 ./ (sc_v1+sc_v2+eps);
		sc_dist = 0.5 * mean( help_sc_dist(:) ); 
	else
		sc_dist = 0.2;
	end

    % punishment for many template point match to one edgel
    uni = unique( label_indeces );
    punish = length(template) / length(uni) - 1;
    if punish > max_scale
        punish = max_scale;
    end
    punish = punish / max_scale;
   
    % combination
    p1 = 1.5;
    p2 = 2;
    p3 = 1.5;
    p4 = 1;
 
    dist = (p1*distance + p2*angle_diff + p3*sc_dist + p4*punish) / (p1+p2+p3+p4);
    
    
    